/* Copyright (C) 2003, Thomas Petazzoni

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA. 
*/

/* Include definitions of the multiboot standard */
#include "multiboot.h"

/* The text video memory starts at address 0xB8000. Odd bytes are the
   ASCII value of the character, even bytes are attribute for the
   preceding character. */
#define VIDEO                   0xB8000

#define LINES   25
#define COLUMNS 80

/* A pointer to this text video memory */
static volatile unsigned char *video;

/* This macro allows to write to an I/O port */
#define outb(value, port)                                       \
  __asm__ volatile (                                            \
        "outb %b0,%w1"                                          \
        ::"a" (value),"Nd" (port)                               \
        )                                                       \

/* The C entry point of our operating system */
void cmain(unsigned long magic, unsigned long addr)
{
  multiboot_info_t *mbi;
  int i;

  /* Initialize the video pointer */
  video = (unsigned char *) VIDEO;

  /* Grub sends us a structure, called multiboot_info_t with a lot of
     precious informations about the system, see the multiboot
     documentation for more information. */
  mbi = (multiboot_info_t *) addr;

  /* Reset the video memory */
  for(i = 0; i < (COLUMNS * LINES * 2); i++)
    video[i] = 0;

  /* Show the 'Test' message */
  video[0] = 'T';
  video[1] = 0x6;
  video[2] = 'e';
  video[3] = 0x6;
  video[4] = 's';
  video[5] = 0x6;
  video[6] = 't';
  video[7] = 0x6;

  /* This is a special hack that is only useful when running the
     operating system under the Bochs emulator. If you compiled Bochs
     with the --enable-e9-hack, then any character printed to the 0xE9
     I/O port is printed to the xterm that is running Bochs. This may
     appear to be a detail, but in fact, this functionnality is *VERY*
     precious for debugging purposes */
  outb('K', 0xE9);

  /* An operating system never ends */
  for(;;);

  return;
}
